DEFAULT_CONFIG = {
	display = {
		width = 1280,
		height = 720,
		fullscreen = false
	},
	game = {
		highScore = 0
	}
}

function love.load()

	--Loading Assets
	ini = require "src.class.ini"
	theme = require "src.theme.theme"
	text = require "src.class.text"
	button = require "src.class.button"
	player = require "src.entity.player"
	platform = require "src.entity.platform"
	game = require "src.state.game"

	IDENTITY = "Flappy juice"
	love.filesystem.setIdentity(IDENTITY)
	CONFIG = DEFAULT_CONFIG

	--Creating/loading config file
	if love.filesystem.exists("config.ini") then
		CONFIG = ini.load("config.ini")
	else
		ini.save("config.ini", CONFIG)
	end

	if not love.filesystem.exists("screenshots") then
		love.filesystem.createDirectory("screenshots")
	end

	love.window.setMode(CONFIG.display.width, CONFIG.display.height, {resizable = true, fullscreen = CONFIG.display.fullscreen})
	love.window.setTitle("Flappy Juice")

	screen = {
		width = love.graphics.getWidth(),
		height = love.graphics.getHeight()
	}

	--Font
	FONT = {
		small = love.graphics.newFont("src/font/CODE Bold.otf", math.floor(screen.height * 0.06)),
		large = love.graphics.newFont("src/font/CODE Bold.otf", math.floor(screen.height * 0.09)),
	}

	--image
	vignetteImage = love.graphics.newImage("src/img/vignette.png")
	vignetteBrightness = 0.4

	--GLOBAL
	GRAVITY = screen.height * 1.8
	OS = love.system.getOS()
	SMOOTH_FACTOR = 6
	SMOOTH_CUTOFF = 0.1
	PLAYER_NORMAL_Y = 0
	PLATFORM_NORMAL_X = {}
	PLATFORM_NORMAL_Y = {}

	--Init
	theme:load()
	theme:apply("default")
	game:load()
end

function love.update(dt)
	game:update(dt)
end

function love.draw()
	game:draw()

	love.graphics.setColor(255, 255, 255, 255 * vignetteBrightness)
	love.graphics.draw(vignetteImage, 0, 0, 0, screen.width / vignetteImage:getWidth(), screen.height / vignetteImage:getHeight())
end

function love.keypressed(key)
	if key == "escape" then 
		if love.keyboard.isDown("lshift") or love.keyboard.isDown("rshift") then
			love.event.push("quit")
		end 
	elseif key == "f2" then
		screenshot()
	elseif key == "f3" then
		local w, h, f = love.window.getMode()
		if f.fullscreen then
			love.window.setMode(CONFIG.display.width, CONFIG.display.height, {fullscreen = false, resizable = true})
			love.resize(CONFIG.display.width, CONFIG.display.height)
			CONFIG.display.fullscreen = false
		else
			local m = love.window.getFullscreenModes()
			table.sort(m, function(a, b) return a.width * a.height < b.width * b.height end)
			love.window.setMode(m[#m].width, m[#m].height, {fullscreen = true})
			love.resize(m[#m].width, m[#m].height, true)
			CONFIG.display.fullscreen = true
		end
	end

	game:keypressed(key)
end

function love.resize(width, height, f)
	f = f or false

	PLAYER_NORMAL_Y = normal(player.y, 0, screen.height)

	for i,v in ipairs(platform.array) do
		PLATFORM_NORMAL_X[i] = normal(v.x, 0, screen.width)
		PLATFORM_NORMAL_Y[i] = normal(v.topY, 0, screen.height)
	end

	screen.width = width
	screen.height = height

	if not f then
		CONFIG.display.width = width
		CONFIG.display.height = height
	end
	--Font
	FONT = {
		small = love.graphics.newFont("src/font/CODE Bold.otf", math.floor(screen.height * 0.06)),
		large = love.graphics.newFont("src/font/CODE Bold.otf", math.floor(screen.height * 0.09)),
	}

	--GLOBAL
	GRAVITY = screen.height * 1.8

	game:resize()
	--game:load()
end

function love.mousepressed(x, y, key)
	game:mousepressed(x, y, key)
end

function love.touchpressed(id, x, y, dx, dy, pressure)
	game:touchpressed(id, x, y, dx, dy, pressure)
end

function love.touchreleased(id, x, y, dx, dy, pressure)
	game:touchreleased(id, x, y, dx, dy, pressure)
end

function love.quit()
	ini.save("config.ini", CONFIG)
end

function setAlpha(a)
	local r, g, b, _ = love.graphics.getColor()
	love.graphics.setColor(r, g, b, a)
end

function AABB(x1,y1,w1,h1, x2,y2,w2,h2)
  return x1 < x2+w2 and
         x2 < x1+w1 and
         y1 < y2+h2 and
         y2 < y1+h1
end

function screenshot()
	local screenshot = love.graphics.newScreenshot()
    screenshot:encode('png', "screenshots/"..os.time() .. '.png')
end

function normal(val, min, max)
	return (val - min) / (max - min)
end
