local button = {}
local bm = {__index = button}

function button.new(text, x, y, width, height, func, hideDirection)
	hideDirection = hideDirection or "bottom"
	local b = {
		text = text,
		x = x,
		y = y,
		dx = x,
		dy = y,
		showY = y,
		width = width,
		height = height,
		func = func,
		hideDirection = hideDirection
	}

	return setmetatable(b, bm)
end

function button.clear()
	bm = {__index = button}
end

function button:setText(t)
	self.text = t
end

function button:update(dt)
	self.dx = self.dx + (self.x - self.dx) * (SMOOTH_FACTOR * dt)
	self.dy = self.dy + (self.y - self.dy) * (SMOOTH_FACTOR * dt)
end

function button:draw()
	love.graphics.setColor(self.color)
	love.graphics.setFont(FONT.small)
	love.graphics.rectangle("fill", self.dx, self.dy, self.width, self.height)
	love.graphics.setColor(game.textColorAccent)
	love.graphics.print(self.text, (self.dx + self.width / 2) - (FONT.small:getWidth(self.text) / 2), (self.dy + (self.height / 2) - ((FONT.small:getAscent() - FONT.small:getDescent()) / 2)))
end

function button:press(x, y)
	if x > self.x and x < self.x + self.width and y > self.y and y < self.y + self.height then
		self.func(self)
		return true
	else return false end
end

function button:hide(instant)
	instant = instant or false
	if self.hideDirection == "bottom" then
		self.y = screen.height * 1.1
	elseif self.hideDirection == "top" then
		self.y = -(screen.height * 0.1)
	end
	if instant then
		self.dy = self.y
	end
end

function button:show(instant)
	instant = instant or false
	self.y = self.showY
	if instant then
		self.dy = self.y
	end
end

return button