local platform = {}

function platform:load()
	self.array = {}

	--Settings
	self:resize()
end

function platform:spawn()
	self.array[#self.array + 1] = {
			x = screen.width - self.width,
			topY = math.random(screen.height * 0.1, (screen.height * 0.9) - self.gap),
			alpha = 255,
			dAlpha = 0,
			cloned = false,
			scored = false
		}
	self.array[#self.array].bottomY = self.array[#self.array].topY + self.gap

	self.array[#self.array].dTopY = 0
	self.array[#self.array].dBottomY = screen.height
end

function platform:resize()
	self.width = math.floor(screen.width * 0.1)
	self.gap = math.floor(screen.height * 0.25)
	self.spawnDistance = math.floor(screen.width * 0.6)
	self.speed = math.floor(screen.width * 0.15)

	if self.array then
		if #self.array > 0 then
			for i,v in ipairs(self.array) do
				v.x = screen.width * PLATFORM_NORMAL_X[i]
				v.topY = screen.height * PLATFORM_NORMAL_Y[i]
				v.dTopY = v.topY
				v.bottomY = v.topY + self.gap
				v.dBottomY = v.bottomY
			end
		end
	end
end

function platform:update(dt)
	for i,v in ipairs(self.array) do
		--slide in
		v.dTopY = v.dTopY + (v.topY - v.dTopY) * (SMOOTH_FACTOR * dt)
		v.dBottomY = v.dBottomY + (v.bottomY - v.dBottomY) * (SMOOTH_FACTOR * dt)
		v.dAlpha = v.dAlpha + (v.alpha - v.dAlpha) * (SMOOTH_FACTOR * dt)


		if not game.paused and game.started and not game.lost then
			v.x = v.x - self.speed * dt

			if not v.cloned then
				if (v.x) < self.spawnDistance then
					self:spawn()
					v.cloned = true
				end
			end

			if v.x < -self.width then
				table.remove(self.array, i)
			end
		end
	end
end

function platform:clear()
	self.array = {}
end

function platform:hideAll()
	for i,v in ipairs(self.array) do
		v.topY = 0
		v.bottomY = screen.height
		v.alpha = 0
	end
end

function platform:draw()
	love.graphics.setColor(self.color)
	for i,v in ipairs(self.array) do
		setAlpha(v.dAlpha)
		love.graphics.rectangle("fill", v.x, 0, self.width, v.dTopY)

		love.graphics.rectangle("fill", v.x, v.dBottomY, self.width, (screen.height - v.dBottomY) )
	end
end

return platform