local game = {}

function game:load()
	--UI Setup
	local verb = "click or press space"
	local escVerb = "press escape"
	if OS == "Android" or OS == "iOS" then
		verb = "tap"
		escVerb = "tap back"
	end
	self.startMessage = text.new(string.format("%s to start", verb), 0, math.floor(screen.height * 0.6), "center", FONT.small, self.textColorAccent, "bottom")
	self.pauseMessage = text.new("paused\n\n"..verb.." to resume", 0, math.floor(screen.height * 0.6), "center", FONT.small, self.textColorAccent, "bottom")
	self.lostText = "GAME OVER\nscore: %d\n\n"..verb.." to restart"
	self.lostMessage = text.new(self.lostText, 0, math.floor(screen.height * 0.4), "center", FONT.small, self.textColorAccent, "bottom")
	self.scoreMessage = text.new("0", 0, math.floor(screen.height * 0.05), "center", FONT.large, self.textColor, "top")
	self.highScoreMessage = text.new("highscore: "..CONFIG.game.highScore, -math.floor(screen.width * 0.01), math.floor(screen.height * 0.93), "right", FONT.small, self.textColor, "bottom")
	self.menuButton = button.new("pause", math.floor(screen.width * 0.01), math.floor(screen.height * 0.9), math.floor(screen.width * 0.2), math.floor(screen.height * 0.09), self.menuPress, "bottom")
	self.exitButton = button.new("exit", math.floor(screen.width * 0.01), math.floor(screen.height * 0.01), math.floor(screen.width * 0.2), math.floor(screen.height * 0.09), self.menuPress, "top")
	
	self.startMessage:hide(true)
	self.pauseMessage:hide(true)
	self.lostMessage:hide(true)
	self.scoreMessage:hide(true)
	self.highScoreMessage:hide(true)
	self.menuButton:hide(true)
	self.exitButton:hide(true)

	self:resize()
	--Screen cover
	self.cover = {
		alpha = 0,
		dAlpha = 0,
		onAlpha = 80
	}

	self.restartDelay = 1
	self.canRestart = true

	self.touched = false

	platform:load()
	player:load()
	game:reset()

	self.glassShader = love.graphics.newShader([[
			
			extern Image mask;
			vec4 effect(vec4 color, Image tex, vec2 tc, vec2 sc) {
				vec4 maskPixel = Texel(mask, tc);

				vec4 pixel = Texel(tex, tc);

				number avg = (pixel.r + pixel.g + pixel.b) / 3.0;
				pixel.r = pixel.r + (avg - pixel.r) * maskPixel.r;
				pixel.g = pixel.g + (avg - pixel.g) * maskPixel.g;
				pixel.b = pixel.b + (avg - pixel.b) * maskPixel.b;

				return pixel;
			}
		]])

	--self.glassShader:send("screen", {screen.width, screen.height})
end

function game:sprint(t)
	self.str = self.str..t.."\n"
end

function game:showCover()
	--self.cover.height = screen.height
	self.cover.alpha = self.cover.onAlpha
end

function game:hideCover()
	--self.cover.height = 0
	self.cover.alpha = 0
end

function game:reset()
	if self.canRestart then
		self.score = 0
		self.scoreMessage:setText(self.score)
		self.restartTick = 0
		self.paused = false
		self.started = false
		self.lost = false
		player:reset()
		platform:clear()
		platform:spawn()
		self.startMessage:show()
		self.lostMessage:hide()
		self.scoreMessage:show()
		self.highScoreMessage:show()
		game:showCover()
	end
end

function game:lose()
	self.lostMessage:setText(string.format(self.lostText, self.score))
	player.alpha = 0
	platform:hideAll()
	self.lostMessage:show()
	self.scoreMessage:hide()
	self.highScoreMessage:hide()
	self:showCover()
	self.canRestart = false
	self.lost = true
	self.menuButton:hide()
end

function game:update(dt)
	--Restart delay
	if self.lost then
		if not self.canRestart then
			self.restartTick = self.restartTick + dt
			if self.restartTick > self.restartDelay then
				self.canRestart = true
			end
		end
	end

	--Cover
	self.cover.dAlpha = self.cover.dAlpha + (self.cover.alpha - self.cover.dAlpha) * (SMOOTH_FACTOR * dt)

	--Text
	self.startMessage:update(dt)
	self.pauseMessage:update(dt)
	self.lostMessage:update(dt)
	self.scoreMessage:update(dt)
	self.highScoreMessage:update(dt)

	self.menuButton:update(dt)
	self.exitButton:update(dt)

	player:update(dt)
	platform:update(dt)

	--Collision
	local colPlatforms = {
		platform.array[1],
		platform.array[2]
	}
	for i,v in ipairs(colPlatforms) do
		if AABB(player.x, player.y, player.size, player.size, v.x, 0, platform.width, v.topY) or AABB(player.x, player.y, player.size, player.size, v.x, v.bottomY, platform.width, (screen.height - v.bottomY)) then
			self:lose()
		end
	end

	--Score count
	for i,v in ipairs(platform.array) do
		if v.x + platform.width < player.x then
			if not v.scored then
				self.score = self.score + 1
				self.scoreMessage:setText(self.score)
				if self.score > CONFIG.game.highScore then
					CONFIG.game.highScore = self.score
					self.highScoreMessage:setText("highscore: "..self.score)
					ini.save("config.ini", CONFIG)
				end
				v.scored = true
			end
		end
	end
end

function game:draw()
	--Creating glass mask
	love.graphics.setCanvas(self.glassMask)
	love.graphics.clear()
	local rgb = (255 / self.cover.onAlpha) * self.cover.dAlpha
	local n = normal(rgb, 0, 255)
	love.graphics.setColor(rgb, rgb, rgb, 255)
	love.graphics.rectangle("fill", 0, 0, screen.width, screen.height)

	--[[
	love.graphics.setColor(255, 255, 255, 255)
	for i,v in ipairs(platform.array) do
		--setAlpha(v.dAlpha)
		love.graphics.rectangle("fill", v.x, v.dTopY, platform.width, v.dBottomY - v.dTopY)
	end
	]]
	self.glassShader:send("mask", self.glassMask)



	love.graphics.setCanvas(self.canvas)

	love.graphics.setColor(self.backgroundColor)
	love.graphics.rectangle("fill", 0, 0, screen.width, screen.height)

	player:draw()
	platform:draw()

	love.graphics.setCanvas()

	love.graphics.setColor(255, 255, 255, 255)
	love.graphics.setShader(self.glassShader)
	love.graphics.draw(self.canvas)
	love.graphics.setShader()

	self.scoreMessage:draw()
	self.highScoreMessage:draw()

	--COVER
	love.graphics.setColor(self.coverColor)
	setAlpha(self.cover.dAlpha)
	love.graphics.rectangle("fill", 0, 0, screen.width, screen.height)

	self.startMessage:draw()
	self.pauseMessage:draw()
	self.lostMessage:draw()

	self.menuButton:draw()
	self.exitButton:draw()

	--love.graphics.setColor(255, 255, 255, 126)
	--love.graphics.draw(self.glassMask)
end

function game:flap()
	if not self.started then
		self.started = true
		self.startMessage:hide()
		self.scoreMessage:show()
		self.highScoreMessage:show()
		self:hideCover()
		self.menuButton:show()
	end
	if self.lost then
		self:reset()
	end
	if self.paused then
		self.paused = false
		self.pauseMessage:hide()
		self:hideCover()
		self.exitButton:hide()
	end

	player:flap()
end

function game:keypressed(key)
	if key == "space" then
		self:flap()
	elseif key == "escape" then
		if self.started and not self.lost then
			if self.paused then
				love.event.push("quit")
			else
				self.paused = true
				self.pauseMessage:show()
				game.exitButton:show()
				self:showCover()
			end
		end
	end
end

function game:mousepressed(x, y, key)
	if key == 1 then
		if self.paused then
			local b = self.exitButton:press(x, y)
			if not b then
				self:flap()
			end
		else
			local b = self.menuButton:press(x, y)
			if not b then
				self:flap()
			end
		end
	end
end

function game:touchpressed(id, x, y, dx, dy, pressure)
	if not self.paused then
		local b = self.menuButton:press(x, y)
		if not b then
			self:flap()
		end
	end
end

function game:touchreleased(id, x, y, dx, dy, pressure)
	self.touched = false
end

function game:resize()
	text.clear()
	button.clear()

	player:resize()
	platform:resize()

	self.glassMask = love.graphics.newCanvas(screen.width, screen.height)
	self.canvas = love.graphics.newCanvas(screen.width, screen.height)

	local verb = "click or press space"
	local escVerb = "press escape"
	if OS == "Android" or OS == "iOS" then
		verb = "tap"
		escVerb = "tap back"
	end
	self.startMessage = text.new(string.format("%s to start", verb), 0, math.floor(screen.height * 0.6), "center", FONT.small, self.textColorAccent, "bottom")
	self.pauseMessage = text.new("paused\n\n"..verb.." to resume", 0, math.floor(screen.height * 0.6), "center", FONT.small, self.textColorAccent, "bottom")
	self.lostText = "GAME OVER\nscore: %d\n\n"..verb.." to restart"
	self.lostMessage = text.new(string.format(self.lostText, self.score or 0), 0, math.floor(screen.height * 0.4), "center", FONT.small, self.textColorAccent, "bottom")
	self.scoreMessage = text.new("0", 0, math.floor(screen.height * 0.05), "center", FONT.large, self.textColor, "top")
	self.highScoreMessage = text.new("highscore: "..CONFIG.game.highScore, -math.floor(screen.width * 0.01), math.floor(screen.height * 0.93), "right", FONT.small, self.textColor, "bottom")
	self.menuButton = button.new("pause", math.floor(screen.width * 0.01), math.floor(screen.height * 0.9), math.floor(screen.width * 0.2), math.floor(screen.height * 0.09), self.menuPress, "bottom")
	self.exitButton = button.new("exit", math.floor(screen.width * 0.01), math.floor(screen.height * 0.01), math.floor(screen.width * 0.2), math.floor(screen.height * 0.09), self.menuPress, "top")
	--Showing/hiding

	if not self.started then
		self.startMessage:show(true)
		self.menuButton:hide(true)
		self.exitButton:hide(true)
	else
		self.startMessage:hide(true)
		self.menuButton:show(true)
	end

	if self.paused then
		self.pauseMessage:show(true)
		self.exitButton:show(true)
	else
		self.pauseMessage:hide(true)
	end

	if self.lost then
		self.lostMessage:show(true)
	else
		self.lostMessage:hide(true)
	end

end

function game.menuPress(b)
	if b.text == "pause" then
		game.paused = true
		game.pauseMessage:show()
		game.exitButton:show()
		game:showCover()
	elseif b.text == "exit" then
		love.event.push("quit")
	end
end

return game